/*
 *  Copyright (C) 2006 by Matthew Spencer
 *
 *  This program is free software; you may redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *  For any other enquiries write to Matthew Spencer, School of Biological Sciences,
 *  University of Liverpool, Crown Street, Liverpool, L69 7ZB, UK.
 *
 *  Or send E-mail to m.spencer@liverpool.ac.uk
 */

#include <stdio.h>
#include <stdlib.h>
#include "array_funcs.h"

#define maxline 1000
#define fail -1
#define success 0


/*structure to hold data*/
typedef struct
{
	int ntimes;/*number of time intervals*/
	int s;/*number of states*/
	double *tset;/*set of sample times excluding 0 (assumed to be the first one)*/
	int *n0;/*initial counts*/
	int **ntrans_cat;/*transition count matrices, concatenated in a row*/
} transdata;


transdata *ReadData (char *filename);
void free_transdata(transdata *td);
transdata *make_transdata(int ntimes, int s);
void printtd(transdata *td);

/*read data from an ASCII file*/
/*first now is ntimes (excludign zero)
second row is s (number of states)
then set of times (excluding zero)
then a block of concatenated transition counts*/
transdata *ReadData (char *filename)
{
	FILE *infile;
	char *myline;
	int ntimes,s,i,j;
	transdata *td;

	/*memory for a line*/
	myline=(char*)calloc(maxline,sizeof(char));
	if(myline==NULL){
		fprintf(stderr,"ERROR: calloc failed");
		exit(EXIT_FAILURE);
	}

	/*open the data file*/
	if( (infile=fopen(filename,"r")) == NULL)
	{
    		fprintf(stderr,"File not found:%s\n",filename);
		exit(EXIT_FAILURE);
  	}

	fscanf(infile,"%d\n",&ntimes);
	fscanf(infile,"%d\n",&s);

	/*get memory for the data*/
	td=make_transdata(ntimes, s);

	/*read tset*/
	for(i=0;i<ntimes;i++) fscanf(infile,"%lf",&td->tset[i]);

	/*read ntrans_cat*/
	for(i=0;i<s;i++){
		for(j=0;j<ntimes*s;j++) fscanf(infile,"%d",&td->ntrans_cat[i][j]);
		fscanf(infile,"\n");
	}

	/*calculate n0*/
	for(j=0;j<s;j++){
		td->n0[j]=0;
		for(i=0;i<s;i++){
			td->n0[j]+=td->ntrans_cat[i][j];
		}
	}

	/*close the data file*/
	fclose(infile);
	free(myline);
	return td;
}

/*allocate memory for a structure containing data*/
transdata *make_transdata(int ntimes, int s)
{
	transdata *td;
	int i;
	
	td=(transdata *)calloc(1,sizeof(transdata));
	if(td==NULL){
		fprintf(stderr,"ERROR: calloc failed");
		exit(EXIT_FAILURE);
	}
	td->ntimes=ntimes;
	td->s=s;
	td->ntrans_cat=(int **)calloc(s,sizeof(int*));
	if(td->ntrans_cat==NULL){
		(void)fprintf(stderr,"ERROR: calloc failed");
		(void)exit(EXIT_FAILURE);
	}

	for(i=0;i<s;i++){
		td->ntrans_cat[i]=(int*)calloc(s*ntimes,sizeof(int));
		if(td->ntrans_cat[i]==NULL){
			fprintf(stderr,"ERROR: calloc failed");
			exit(EXIT_FAILURE);
		}
	}

	td->tset=(double *)calloc(s,sizeof(double));
	if(td->tset==NULL){
		fprintf(stderr,"ERROR: calloc failed");
		exit(EXIT_FAILURE);
	}
	td->n0=(int *)calloc(s,sizeof(int));
	if(td->n0==NULL){
		fprintf(stderr,"ERROR: calloc failed");
		exit(EXIT_FAILURE);
	}
	return td;
}

/*free memory for structure containing data*/
void free_transdata(transdata *td)
{
	int i;
	for(i=0;i<td->s;i++) free(td->ntrans_cat[i]);
	free(td->ntrans_cat);
	free(td->tset);
	free(td->n0);
	free(td);
}

/*print out the data*/
void printtd(transdata *td)
{
	int i,j;

	printf("%d times, %d states\n",td->ntimes,td->s);
	printf("Sample times: ");
	for(i=0;i<td->ntimes;i++) printf("%f ",td->tset[i]);
	printf("\n");
	printf("Transition counts:\n");
	for(i=0;i<td->s;i++){
		for(j=0;j<(td->s * td->ntimes);j++){
			printf("%d ",td->ntrans_cat[i][j]);
		}
		printf("\n");
	}
	printf("Initial counts:\n");
	for(i=0;i<td->s;i++){
		printf("%d\n",td->n0[i]);
	}
	printf("\n\n");
}
